/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.analysis;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import com.ibm.wala.classLoader.CallSiteReference;
import com.ibm.wala.classLoader.IClass;
import com.ibm.wala.classLoader.IMethod;
import com.ibm.wala.classLoader.ShrikeCTMethod;
import com.ibm.wala.ipa.callgraph.propagation.InstanceKey;
import com.ibm.wala.shrikeBT.IInstruction;
import com.ibm.wala.shrikeBT.InvokeInstruction;
import com.ibm.wala.shrikeCT.InvalidClassFileException;
import com.ibm.wala.ssa.DefUse;
import com.ibm.wala.ssa.IR;
import com.ibm.wala.ssa.SSAAbstractInvokeInstruction;
import com.ibm.wala.ssa.SSAInstruction;
import com.ibm.wala.types.MethodReference;
import com.ibm.wala.util.graph.Graph;
import com.ibm.wala.util.intset.IntSet;
import com.ibm.wala.util.warnings.WalaException;

/**
 * 
 * @author Stas Negara
 * 
 */
public class AnalysisHelper {

	private static final String PRIMORDIAL_CLASSLOADER_NAME = "Primordial";

	private static final String INIT_METHOD_NAME = "<init>";
	private static final String CLASS_INIT_METHOD_NAME = "<clinit>";
	private static final String SPECIAL_INVOCATION_MODE = "SPECIAL";

	public static boolean isJDKClass(IClass klass) {
		return klass.getClassLoader().getName().toString().equals(PRIMORDIAL_CLASSLOADER_NAME);
	}

	public static Set<SSAInstruction> getUseInstructions(IR ir, int valueNumber) {
		Set<SSAInstruction> useInstructions = new HashSet<SSAInstruction>();
		DefUse defUse = new DefUse(ir);
		Iterator<SSAInstruction> useInstructionsIterator = defUse.getUses(valueNumber);
		while (useInstructionsIterator.hasNext()) {
			useInstructions.add(useInstructionsIterator.next());
		}
		return useInstructions;
	}

	public static int getBytecodeIndexForCallSite(IR ir, CallSiteReference callSiteReference) {
		IntSet callInstructionIndices = ir.getCallInstructionIndices(callSiteReference);
		if (callInstructionIndices.size() > 1) {
			throw new RuntimeException("Several indices for call site: " + callSiteReference);
		}
		return callInstructionIndices.intIterator().next();
	}

	public static boolean isInitInvocation(SSAAbstractInvokeInstruction invokeInstruction) {
		return invokeInstruction.isSpecial() && isInitMethod(invokeInstruction.getDeclaredTarget());
	}

	public static boolean isInitInvocation(InvokeInstruction invokeInstruction) {
		return invokeInstruction.getInvocationModeString().equals(SPECIAL_INVOCATION_MODE) && isInitMethodName(invokeInstruction.getMethodName());
	}

	public static boolean isInitMethod(MethodReference methodReference) {
		return isInitMethodName(methodReference.getName().toString());
	}

	public static boolean isInitMethod(IMethod method) {
		return isInitMethodName(method.getName().toString());
	}

	private static boolean isInitMethodName(String methodName) {
		return methodName.equals(INIT_METHOD_NAME);
	}

	public static boolean isClassInitMethod(IMethod method) {
		return isClassInitMethodName(method.getName().toString());
	}

	private static boolean isClassInitMethodName(String methodName) {
		return methodName.equals(CLASS_INIT_METHOD_NAME);
	}

	public static IInstruction[] getShrikeInstructions(ShrikeCTMethod shrikeMethod) {
		IInstruction[] instructions = null;
		try {
			instructions = shrikeMethod.getInstructions();
		} catch (InvalidClassFileException e) {
			e.printStackTrace();
		}
		return instructions;
	}

	@SuppressWarnings("rawtypes")
	public static void displayGraph(Graph graph) throws WalaException {
		//		final SWTTreeViewer v = new SWTTreeViewer();
		//		v.setGraphInput(graph);
		//		v.setRootsInput(InferGraphRoots.inferRoots(graph));
		//		v.run();
		//		v.getApplicationWindow();
	}

	/**
	 * Turned off immutability detection in order to have pure analysis results
	 * 
	 * @param instanceKey
	 * @return
	 */
	public static boolean isImmutableInstanceKey(InstanceKey instanceKey) {
		//		String className = instanceKey.getConcreteType().getName().toString();
		//		if (className.equals("Ljava/lang/Character") || className.equals("Ljava/lang/Byte") || 
		//			className.equals("Ljava/lang/Short") || className.equals("Ljava/lang/Integer") || 
		//			className.equals("Ljava/lang/Long") || className.equals("Ljava/lang/Float") || 
		//			className.equals("Ljava/lang/Double") || className.equals("Ljava/lang/Boolean") || 
		//			className.equals("Ljava/lang/String") || className.equals("Losl/manager/ActorName")){
		//			return true;
		//		}
		return false;
	}

}
