/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.analysis.actorfoundry;

import java.util.HashMap;
import java.util.Map;

import com.ibm.wala.classLoader.CallSiteReference;
import com.ibm.wala.classLoader.IClass;
import com.ibm.wala.classLoader.IMethod;
import com.ibm.wala.ipa.callgraph.impl.AbstractRootMethod;
import com.ibm.wala.ipa.cha.IClassHierarchy;
import com.ibm.wala.ssa.SSAAbstractInvokeInstruction;
import com.ibm.wala.types.TypeReference;

import edu.illinois.soter.analysis.FieldSensitiveEntrypoint;

/**
 * 
 * @author Stas Negara
 * 
 */
public class SingleInstanceEntrypoint extends FieldSensitiveEntrypoint {

	private static final Map<TypeReference, Integer> callerInstances = new HashMap<TypeReference, Integer>();

	public SingleInstanceEntrypoint(IClass klass, IMethod method, IClassHierarchy cha) {
		super(method, cha);
		//Make *this* to be of *klass* type rather than type, which declared the method (default behavior)
		setParameterTypes(0, new TypeReference[] { klass.getReference() });
	}

	/**
	 * All entry point methods of a particular class are called on the same
	 * instance of this class. Used to simulate messages for an Actor.
	 */
	@Override
	public SSAAbstractInvokeInstruction addCall(AbstractRootMethod rootMethod) {
		CallSiteReference site = makeSite(rootMethod.getStatements().length);
		if (site == null) {
			return null;
		}
		int paramValues[] = new int[getNumberOfParameters()];
		if (paramValues.length > 0) {
			TypeReference[] typeReferences = getParameterTypes(0);
			if (typeReferences.length == 0 || typeReferences.length > 1) {
				throw new RuntimeException("None or multiple types for the receiver in entry point: " + this);
			}
			Integer thisValueNumber = callerInstances.get(typeReferences[0]);
			if (thisValueNumber == null) {
				thisValueNumber = makeArgument(rootMethod, 0);
				if (thisValueNumber == -1) {
					//there was a problem
					return null;
				}
				callerInstances.put(typeReferences[0], thisValueNumber);
			}
			paramValues[0] = thisValueNumber;
			for (int i = 1; i < paramValues.length; i++) {
				paramValues[i] = makeArgument(rootMethod, i);
				if (paramValues[i] == -1) {
					//there was a problem
					return null;
				}
			}
		}
		return rootMethod.addInvocation(paramValues, site);
	}

}
