/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.analysis.contextselectors;

import com.ibm.wala.classLoader.CallSiteReference;
import com.ibm.wala.classLoader.IMethod;
import com.ibm.wala.ipa.callgraph.CGNode;
import com.ibm.wala.ipa.callgraph.Context;
import com.ibm.wala.ipa.callgraph.ContextSelector;
import com.ibm.wala.ipa.callgraph.impl.Everywhere;
import com.ibm.wala.ipa.callgraph.propagation.InstanceKey;
import com.ibm.wala.ipa.callgraph.propagation.cfa.CallerSiteContext;
import com.ibm.wala.ipa.callgraph.propagation.cfa.CallerSiteContextPair;

/**
 * Customized to be more precise for particular classes. This is a context
 * selector that adds one level of calling context to a base context selector.
 * 
 * @author Stas Negara
 * 
 */
public class CustomizedOneLevelSiteContextSelector implements ContextSelector {

	private final ContextSelector baseContextSelector;

	public CustomizedOneLevelSiteContextSelector(ContextSelector baseContextSelector) {
		assert baseContextSelector != null;
		this.baseContextSelector = baseContextSelector;
	}

	public Context getCalleeTarget(CGNode callerNode, CallSiteReference siteReference, IMethod calleeMethod, InstanceKey receiverInstanceKey) {
		if (callerNode.getMethod().getDeclaringClass().getName().getClassName().toString().equals("HelloActor")) {
			Context baseContext = baseContextSelector.getCalleeTarget(callerNode, siteReference, calleeMethod, receiverInstanceKey);
			if (baseContext.equals(Everywhere.EVERYWHERE)) {
				return new CallerSiteContext(callerNode, siteReference);
			} else {
				return new CallerSiteContextPair(callerNode, siteReference, baseContext);
			}
		} else {
			return baseContextSelector.getCalleeTarget(callerNode, siteReference, calleeMethod, receiverInstanceKey);
		}
	}

}
