/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.messagedata;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import com.ibm.wala.classLoader.CallSiteReference;
import com.ibm.wala.classLoader.ShrikeCTMethod;
import com.ibm.wala.ipa.callgraph.CGNode;

/**
 * 
 * @author Stas Negara
 * 
 */
public class MessageInvocation {

	private final CGNode cgNode;
	private final CallSiteReference callSiteReference;
	private final LinkedList<MessageArgument> arguments = new LinkedList<MessageArgument>();

	public MessageInvocation(CGNode cgNode, CallSiteReference callSiteReference) {
		this.cgNode = cgNode;
		this.callSiteReference = callSiteReference;
	}

	public void addArgument(int transformationBytecodeIndex, Set<Object> argumentItems) {
		arguments.add(new MessageArgument(transformationBytecodeIndex, argumentItems));
	}

	public void addFirstArgument(int transformationBytecodeIndex, Set<Object> argumentItems) {
		arguments.addFirst(new MessageArgument(transformationBytecodeIndex, argumentItems));
	}

	public Iterator<MessageArgument> getArgumentsIterator() {
		return arguments.iterator();
	}

	public CGNode getCGNode() {
		return cgNode;
	}

	public String toString() {
		StringBuffer returnValue = new StringBuffer();
		returnValue.append("MessageInvocation:\n");
		returnValue.append("CGNode: ").append(cgNode).append("\n");
		returnValue.append("CallSiteReference: ").append(callSiteReference.toString()).append("\n");
		ShrikeCTMethod method = (ShrikeCTMethod) cgNode.getMethod();
		returnValue.append("Line number: ").append(method.getLineNumber(callSiteReference.getProgramCounter())).append("\n");
		for (MessageArgument argument : arguments) {
			returnValue.append("Argument:\n").append(argument).append("\n");
		}
		return returnValue.toString();
	}

	/**
	 * Checking own arguments for cases like "my_method(a, a)" - same item is
	 * passed for two different arguments.
	 */
	public void selfCheck() {
		for (int i = 0; i < arguments.size() - 1; i++) {
			MessageArgument argument = arguments.get(i);
			for (int j = i + 1; j < arguments.size(); j++) {
				argument.check(arguments.get(j));
			}
		}
	}

	/**
	 * Checks if any arguments of this MessageInvocation have common items with
	 * the provided set of live items, in which case these arguments should be
	 * passed by value.
	 * 
	 * @param messageInvocation
	 */
	public void check(Set<Object> liveItems) {
		selfCheck();
		for (int i = 0; i < arguments.size(); i++) {
			arguments.get(i).check(liveItems);
		}
	}

}
